# encoding: utf-8

require 'logstash/inputs/base'
require 'logstash/namespace'
require 'stud/interval'
require 'socket' # for Socket.gethostname
require 'rest-client'
require 'json'

class CloudflareAccess
  DEFAULT_FIELDS = %w(CacheCacheStatus CacheResponseBytes CacheResponseStatus ClientASN ClientCountry ClientDeviceType ClientIP ClientIPClass ClientRequestBytes ClientRequestHost ClientRequestMethod ClientRequestProtocol ClientRequestReferer ClientRequestURI ClientRequestUserAgent ClientSSLCipher ClientSSLProtocol ClientSrcPort EdgeColoID EdgeEndTimestamp EdgePathingStatus EdgeResponseBytes EdgeResponseCompressionRatio EdgeResponseStatus EdgeStartTimestamp OriginIP OriginResponseBytes OriginResponseHTTPExpires OriginResponseHTTPLastModified OriginResponseStatus OriginResponseTime RayID WAFAction WAFRuleID ZoneID).freeze

  attr_accessor :auth_email, :auth_key, :domain, :fields, :metadata_file
  def initialize(arguments)
    @auth_email = arguments[:auth_email]
    @auth_key = arguments[:auth_key]
    @domain = arguments[:domain]
    @fields = arguments[:fields] || DEFAULT_FIELDS
    @metadata_file = arguments[:metadata_file]
  end

  def start_time
    if File.exist?(metadata_file)
      start_time = Time.parse(JSON.parse(File.read(metadata_file))['start_time']).to_datetime.rfc3339
    else
      start_time = (Time.now - (15 * 60)).to_datetime.rfc3339
    end
    start_time
  end

  def end_time
    @end_time ||= (Time.now - (5 * 60)).to_datetime.rfc3339
  end

  def update_metadata_file(key, value)
    key = key.to_s
    meta_data = {}

    if File.exist?(metadata_file)
      begin
        meta_data = JSON.parse(File.read(metadata_file))
      rescue
      end
    end

    meta_data[key] = value

    File.open(metadata_file, 'w+') { |file| file.write(meta_data.to_json) }
  end

  def logs
    results = RestClient.get("https://api.cloudflare.com/client/v4/zones/#{domain}/logs/received?start=#{start_time}&end=#{end_time}&fields=#{fields.join(',')}", 'X-Auth-Email' => auth_email, 'X-Auth-Key' => auth_key)
    results.body.split("\n").collect { |raw_log| JSON.parse(raw_log) }
  end
end

class LogStash::Inputs::CloudflareLogs < LogStash::Inputs::Base
  config_name 'cloudflare-logs'

  # If undefined, Logstash will complain, even if codec is unused.
  default :codec, 'json'

  config :auth_email, validate: :string, required: true
  config :auth_key, validates: :string, required: true
  config :domain_key, validates: :string, required: true
  config :metadata_file, validates: :string, default: '/etc/logstash/cf_metadata.json'

  config :interval, validate: :number, default: 600

  def register
    @host = Socket.gethostname
  end # def register

  def cloudflare_access
    CloudflareAccess.new(auth_key: @auth_key,
                         auth_email: @auth_email,
                         domain: @domain_key,
                         metadata_file: @metadata_file)
  end

  def process_logs(queue)
    cloudflare_access.logs.each do |log|
      event = LogStash::Event.new(log)
      decorate(event)
      queue << event
    end
  end

  def run(queue)
    # we can abort the loop if stop? becomes true
    until stop?
      process_logs(queue)

      # because the sleep interval can be big, when shutdown happens
      # we want to be able to abort the sleep
      # Stud.stoppable_sleep will frequently evaluate the given block
      # and abort the sleep(@interval) if the return value is true
      Stud.stoppable_sleep(@interval) { stop? }
    end # loop
  end # def run

  def stop
    # nothing to do in this case so it is not necessary to define stop
    # examples of common 'stop' tasks:
    #  * close sockets (unblocking blocking reads/accepts)
    #  * cleanup temporary files
    #  * terminate spawned threads
  end
end # class LogStash::Inputs::CloudflareLogs
